# Copyright 2000, 2005 Sergey Poznyakoff
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

# State map:
#   0 -- Initial
#   1 -- in zone
#   2 -- in zone sub-statement
#   3 -- in non-secondary block (after ##:NOSECONDARY and before ##:SECONDARY)

BEGIN {
      ignored["."] = 1;
      ignored["0.0.127.IN-ADDR.ARPA"] = 1;
      if (!MASTERS) {
	 print "MASTERS not defined"
	 exit 1
      }
      n = split(MASTERS,a,";")
      MASTERS = ""
      for (i = 1; i <= n; i++) {
	  MASTERS = MASTERS "\t\t" a[i] ";\n"
      }
      print "## This file is automatically generated by ndc.awk"
      print "## Do not edit"
}
      
function debug(s) {}

function lineinfo(n,    rem_source_name) {
    if (MACHINE)
       rem_source_name = MACHINE ":" source_name
    else
       rem_source_name = source_name
    print "#line " n + offset " \"" rem_source_name "\""
}

/# [0-9][0-9]* ".[^"]*"$/ {
    source_name = substr($3,2,length($3)-2)
    offset = $2 - NR - 1
    debug("OFFSET=" offset)
    lineinfo(NR)
    next
}

state == 0 && /^zone/ {
    state = 1
    debug("Processing " $2)
    zone[$1] = substr($2,2,length($2)-2)
    zone_line = NR
    next
}

state == 0 && /##:NOSECONDARY/ {
    state = 3
    next
}

state == 0 && /#.*/ {
    print
    next  
}

state == 0 { next }

state == 1 && /.*{/ {
    stmt = $1
    zone[$1] = "{"
    state = 2
    next
}

state == 1 && /#.*/ {
    next		
}

state == 1 && /};/ {
    # Flush the zone
    if (!ignored[zone["zone"]]) {
       lineinfo(zone_line)
       print comment;
       print "zone \"" zone["zone"] "\" {"
       if (zone["type"] == "master") {
	  zone["type"] = "slave";
	  zone["file"] = "\"" zone["zone"] ".s\""
	  zone["masters"] = "{\n" MASTERS "\t}"
       }
       delete zone["zone"]
       for (stmt in zone) {
	   if (stmt != "allow-update")
	      print "\t" stmt " " zone[stmt] ";"
       }	 
       print "};"
    }    
    delete zone
    comment = ""
    state = 0
    next
}

state == 1 && NF == 2 && /.*;/ {
    zone[$1] = substr($2,1,length($2)-1);
}

## sub-statement in zone block
state == 2 && /};/ {
    debug("STMT " stmt)
    zone[stmt] = zone[stmt] "\n\t}"
    state = 1;
    next
}

state == 2 {
    zone[stmt] = zone[stmt] "\n" $0;
}

state == 3 && /##:SECONDARY/ {
    state = 0
}    
